/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2020.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { SGraphItem } from "@saga-web/graph/lib";
import { SColor, SPainter, SPoint, SRect } from "@saga-web/draw/lib";
import { Anchor } from "../types/Anchor";

/**
 * 关系item-折线画法-横平竖直
 *
 * @author  郝建龙
 */
export class AnchorItem extends SGraphItem {
    /** 所有数据    */
    data: Anchor | null = null;
    /** 相对于父级的x坐标   */
    X: number;
    /** 相对于父级的y坐标   */
    Y: number;
    /** 宽度  */
    Width: number = 10;
    /** 高度  */
    Height: number = 10;
    /** 是否被连接   */
    isConnected: boolean = false;
    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      Relation数据
     */
    constructor(parent: SGraphItem | null, data: Anchor) {
        super(parent);
        this.data = data;
        this.X = data.Pos.X;
        this.Y = data.Pos.Y;
        if (data.Width) {
            this.Width = data.Width;
        }
        if (data.Height) {
            this.Height = data.Height;
        }
        if (parent) {
            this.zOrder = parent.zOrder + 1;
        }
    } // Constructor

    /**
     * Item绘制操作
     *
     * @param   painter       painter对象
     */
    onDraw(painter: SPainter): void {
        painter.pen.lineWidth = 2;
        painter.brush.color = this.isConnected
            ? new SColor("#409EFF")
            : SColor.White;
        painter.pen.color = new SColor("#409EFF");
        painter.drawRect(
            this.X - this.Width / 2,
            this.Y - this.Height / 2,
            this.Width,
            this.Height
        );
    } // Function onDraw()
} // Class RelationItem
